/*------------------------------------------------------------------------------
--  This file is a part of the SocExplorer Software
--  Copyright (C) 2012, Plasma Physics Laboratory - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Alexis Jeandet
--                     Mail : alexis.jeandet@lpp.polytechnique.fr
----------------------------------------------------------------------------*/
#ifndef DSU3PLUGIN_H
#define DSU3PLUGIN_H
#include "dsu3pluginui.h"
#include <QMenuBar>
#include <QMenu>
#include <QAction>
#include <QMainWindow>
#include <QList>
#include "elfparser.h"
#include <elffile.h>
#include <socexplorerplugin.h>
#include <socexplorerengine.h>

typedef QList<unsigned int> uIntlist;

class dsu3plugin : public socexplorerplugin
{
    Q_OBJECT
public:
    explicit dsu3plugin(QWidget *parent = 0);
    ~dsu3plugin();
    bool memSet(unsigned int address, int value, unsigned int count);
public slots:
    void openFile();
    void openFile(QString fileName);
    bool configureTarget();
    bool cacheDisable();
    bool cacheEnable();
    bool cacheIsEnable();
    bool setCacheEnable(bool enabled);
    bool flashTarget();
    void run();
    void stop();
    void toggleRun();
    //added for python backward compatibility
    void loadFile(){this->flashTarget();}
    bool dumpSymbol(const QString& symbolName,QString file,const QString& format);
    QVariantList readSymbol(const QString& symbolName);
signals:
    void updateInfo(ElfFile* elfFile);
private:
    void WriteRegs(uIntlist Values, unsigned int address);
    unsigned int ReadReg(unsigned int address);
    void writeSection(int index);
    void writeSection(const QString& name);
    dsu3pluginui* UI;
    ElfFile* elfFile;
    bool running;
};

#endif // DSU3PLUGIN_H
