/*------------------------------------------------------------------------------
--  This file is a part of the SocExplorer Software
--  Copyright (C) 2014, Plasma Physics Laboratory - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Alexis Jeandet
--                     Mail : alexis.jeandet@member.fsf.org
----------------------------------------------------------------------------*/
#ifndef SPWTCPPACKETSERVER_H
#define SPWTCPPACKETSERVER_H

#include <QWidget>
#include <abstractspwbridge.h>
#include <QTcpServer>
#include <QList>
#include <QTcpSocket>
#include <incomingpacketparser.h>

namespace Ui {
class SpwTcpPacketServer;
}

class SpwTcpPacketServer : public QWidget
{
    Q_OBJECT

public:
    explicit SpwTcpPacketServer(QWidget *parent = 0);
    ~SpwTcpPacketServer();

    void setBridge(abstractSpwBridge* bridge);
signals:
    void sendSPWPacket(char* packet,int size);
public slots:
    void pushPacket(char* packet,int size);
    void toggleServer();
    void connectServer();
    void disconnectServer();
    void setServerPort(qint32 port);
    void setServerSetIP(QString ip);
    void newConnection();
    void parseIncomingPacket();
    void sendSPWPacketUsingSpaceWire(QByteArray data );
    void readReady();
    void resetStatististics();
private:
    void updateHostIP();
    void onePacketReceived();
    void onePacketTransmitted();
    Ui::SpwTcpPacketServer *ui;
    abstractSpwBridge* p_bridge;
    QTcpServer* p_server;
    QList<QTcpSocket*> connectedClients;
    unsigned int receivedPackets;
    unsigned int transmittedPackets;
    IncomingPacketParser *incomingPacketParser;
};

#endif // SPWTCPPACKETSERVER_H
