/*------------------------------------------------------------------------------
--  This file is a part of the SocExplorer Software
--  Copyright (C) 2014, Plasma Physics Laboratory - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Alexis Jeandet
--                     Mail : alexis.jeandet@member.fsf.org
----------------------------------------------------------------------------*/
#ifndef STARDUNDEEGUI_H
#define STARDUNDEEGUI_H

#include <QWidget>

#include "fakestardundeespwusbgui.h"

namespace Ui {
    class StarDundeeUI;
}

class StarDundeeGUI : public QWidget
{
    Q_OBJECT
public:
    explicit StarDundeeGUI(QWidget *parent = 0);
    int getBrickSelection();
    int getLinkNumberSelection();
    QString getLinkSpeedSelection();
    QString getSourceAddress();
    QString getDestinationAddress();
    QString getDestinationKey();
    QString getRmapTimeout();
    QString getTimecodeFrequency();
    bool getStartSendingTimecodes();
    bool isBrickSetAsAnInterface();
signals:
    void brickSelectionChanged(int brickIndex);
    void linkNumberSelectionChanged(int linkIndex);
    void linkSpeedSelectionChanged(const QString & linkSpeed);
    void destinationKeyChanged(const QString & destKey);
    void sourceLogicalAddressChanged( QString );
    void rmapAddressChanged(const QString & address);
    void rmapKeyChanged(const QString & key);
    void rmapTimeoutChanged(const QString & timeout);
    void timecodeFrequencyChange(const QString & timeout);
    void connectClicked();
    void brickModeChanged(bool);
    void startSendingTimecode(bool);
public slots:
    void showDriverStatus();
    void lock(bool lock);
    void updateAvailableBrickCount(int count);
    void selectBrick(int brickIndex);
    void selectLinkNumber(int linkIndex);
    void selectLinkSpeed(int linkSpeed);
    void setDestinationKey(const QString & destKey);
    void setSourceAddress(const QString & address);
    void setDestinationAddress(const QString & address);
    void setTimecodeFrequency(double frequency);
    void setStartSendingTimecodes( bool onOff );
    void setRmapTimeout(const QString & timeout);
    int getAvailableBrickCount( void );
    void setBrickAsAnInterface(bool interfaceMode);
    void setBrickAsARouter( bool interfaceMode );
    void resetBytesPacketsStatistics( void );
    void resetStatistics( void );
    void updateNbReceivedBytesFromSpw( unsigned int nbBytes);
    void updateNbTransmittedBytesToSpw( unsigned int nbBytes);
    void updateCCSDSPacketTransmittedToSpw( void );
    unsigned int getNbPacketsTransmittedToSpw( void );
    unsigned int getNbCCSDSPacketsTransmittedToSpw( void );
private:
    Ui::StarDundeeUI *ui;
    unsigned int nbBytesReceivedFromSpw;
    unsigned int nbBytesTransmittedToSpw;
    unsigned int nbPacketsReceivedFromSpw;
    unsigned int nbPacketsTransmittedToSpw;
    unsigned int nbCCSDSPacketsTransmittedToSpw;
    FakeStardundeeSpwUsbGui driverStatus;
};

#endif // STARDUNDEEGUI_H
