/*------------------------------------------------------------------------------
--  This file is a part of the SocExplorer Software
--  Copyright (C) 2014, Plasma Physics Laboratory - CNRS
--
--  This program is free software; you can redistribute it and/or modify
--  it under the terms of the GNU General Public License as published by
--  the Free Software Foundation; either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU General Public License
--  along with this program; if not, write to the Free Software
--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-------------------------------------------------------------------------------*/
/*--                  Author : Alexis Jeandet
--                     Mail : alexis.jeandet@member.fsf.org
----------------------------------------------------------------------------*/
#ifndef STARDUNDEESPW_USB_H
#define STARDUNDEESPW_USB_H

#include <QObject>
//#include <spw_usb_api.h>
//#include <spw_config_library.h>

#include "fakestardundeespwusb_lib.h"
#include <socexplorerplugin.h>
#include <abstractspwbridge.h>
#include <QThread>
#include <QMutex>
#include <QSemaphore>
#include <QGridLayout>
#include <QPushButton>
#include <QComboBox>
#include <QLabel>
#include "stardundeegui.h"

#define BRICK_IS_SET_AS_AN_INTERFACE true
#define BRICK_IS_SET_AS_A_ROUTER false

class stardundeeSPW_USB_Manager: public abstractSpwManager
{
    Q_OBJECT
public:
    explicit stardundeeSPW_USB_Manager(socexplorerplugin *plugin = 0,QObject* parent=0);
    ~stardundeeSPW_USB_Manager();
    void run();
    bool connectBridge();
    bool connectBridgeAsInterface();
    bool connectBridgeAsRouter();
    void initDialog( void );
    unsigned char setRoutingTableEntry(int tableEntry, U32 dwOutputPorts, char bDelHead);
    unsigned int getRoutingTableEntry(int tableEntry);
    void initializeTimecodeGeneration();
    void setTimecodeFrequency(double requestedFrequency);
    unsigned int getLinkStatus(unsigned char link);
    bool disconnectBridge();
    bool sendPacket(char* packet,int size);

signals:
    void updateAvailableBrickCount(int count);
    void emitPacket(char* packet,int size);
    void bytesReceivedFromSpw( unsigned int );
    void bytesTransmittedToSpw( unsigned int);
    void ccsdsPacketTransmittedToSpw( void );

public slots:
    void sendTimecodePeriodically( bool onOff );
    void sendOneTimecode(unsigned char nTimein);
    int getLinkNumber( void );

private:
    //QSemaphore* RMAP_AnswersSem;
    star_device_handle hDevice;

    QLabel *starDundeeStatusQueryDialogLabel;
    QPushButton *starDundeeStatusQueryRetryButton;
    QPushButton *starDundeeStatusQueryAbortButton;
    QLabel *statusLink1;
    QLabel *statusLink2;
    QPushButton *starDundeeStatusQueryContinueButton;
    QDialog *starDundeeStatusQueryDialog;
    QGridLayout *starDundeeStatusQueryDialogLayout;

public:
    int selectedBrick;
    int brickList;
    int linkSpeed;
    double timecodeFrequency;
    bool interfaceMode; // 1 => interface mode, 0 => router mode
};

class stardundeeSPW_USB : public abstractSpwBridge
{
    Q_OBJECT
public:
    explicit stardundeeSPW_USB(socexplorerplugin *parent = 0);
    ~stardundeeSPW_USB();

signals:

    void setRmapTimeout(const QString & timeout);
    void SelectBrick(int brickIndex);
    void SelectLinkNumber(int linkIndex);
    void SelectLinkSpeed(int linkSpeed);
    void SetDestinationKey(const QString & destKey);
    void SetSourceAddress(const QString & address);
    void SetDestinationAddress(const QString & address);
    void SetRmapTimeout(const QString & timeout);
    void SetTimecodeFrequency( double );
    void SetBrickAsAnInterface( bool );
    void SetBrickAsARouter( bool );
    int GetAvailableBrickCount( void );
    unsigned int GetNbPacketsTransmittedToSpw( void );
    unsigned int GetNbCCSDSPacketsTransmittedToSpw( void );
    void BytesReceivedFromSpw( unsigned int );
    void BytesTransmittedToSpw( unsigned int );
    void CCSDSPacketTransmittedToSpw( void );
    void StartSendingTimecodes( bool );
    void SendOneTimecode( unsigned char );
    int GetLinkNumber();

public slots:
    void toggleBridgeConnection();
    bool connectBridge();
    bool disconnectBridge();
    int  pushRMAPPacket(char* packet,int size);
    unsigned int Write(unsigned int *Value,unsigned int count, unsigned int address=0);
    unsigned int Read(unsigned int *Value,unsigned int count, unsigned int address=0);
    void brickSelectionChanged(int brickIndex);
    void linkNumberSelectionChanged(int linkIndex);
    void linkSpeedSelectionChanged(const QString & linkSpeed);
    void sourceLogicalAddressChanged(const QString & destKey);
    void destinationAddressChanged(const QString & rmapaddress);
    void brickModeChanged( bool interfaceMode );
    void destinationKeyChanged(const QString & key);
    void rmapTimeoutChanged(const QString & timeout);
    void sendPacketComingFromTCPServer(char *packet, int size);
    void timecodeFrequencyChanged(const QString &frequency);
    void startSendingTimecodes( bool onOff );
private:
    void makeGUI(socexplorerplugin *parent);
    stardundeeSPW_USB_Manager* manager;
};

#endif // STARDUNDEESPW_USB_H
